import sys, os.path, MenuMaker, Prophet




from Keywords import Keyword as Kw, Set as KwS




from optparse import OptionParser as _OP




from MenuMaker import msg, warn, fatal




writeToStdout	= False # Output the generated menu to stdout instead of the menu file
forceWrite		= False # Overwrite existing files
noDesktop		= False # Do not scan for .desktop files
noLegacy		= False # Do not scan for the legacy apps




fronts = {
	Kw("IceWM")			: KwS("IceWM"),
	Kw("BlackBox")		: KwS("BlackBox"),
	Kw("FluxBox")		: KwS("FluxBox"),
	Kw("XFce4")			: KwS("XFce", "XFce4"),
	Kw("WindowMaker")	: KwS("WindowMaker", "WMaker")
}




vStr = "be verbose"
def vOpt(option, opt, value, parser) :
	MenuMaker.verbose = Prophet.verbose = True




fStr = "overwrite existing files"
def fOpt(option, opt, value, parser) :
	global forceWrite
	forceWrite = True




cStr = "dump the menu to stdout instead of file"
def cOpt(option, opt, value, parser) :
	global writeToStdout
	writeToStdout = True




iStr = "produce the output suitable for inclusion into the custom menu; MUST be used in conjunction with -c"
def iOpt(option, opt, value, parser) :
	MenuMaker.writeFullMenu = False




rStr = "always retain the path to executable (by default, the path is omitted whenever possible)"
def rOpt(option, opt, value, parser) :
	Prophet.omitExecPath = False




xdStr = "skip .desktop scanning"
def xdOpt(option, opt, value, parser) :
	global noDesktop
	noDesktop = True




xlStr = "skip legacy scanning"
def xlOpt(option, opt, value, parser) :
	global noLegacy
	noLegacy = True




sKwS = KwS(*Prophet.skip.keys())
sStr = "skip specific categories (comma-separated list)"
def sOpt(option, opt, value, parser) :
	for x in value.split(",") :
		if not len(x) :
			continue
		try :
			x = Kw(x)
			Prophet.skip[x] = True
		except KeyError :
			raise optparse.OptionValueError("wrong argument %s for the option %s" % (x, opt))




class OP(_OP) :
	def print_help(self) :
		sys.stdout.write("This is %s %s, a 100%% Python heuristics-driven menu generator\n" % (MenuMaker.pkgName, MenuMaker.pkgVer))
		sys.stdout.write("%s %s %s <%s>\n\n" % (MenuMaker.pkgHome, MenuMaker.copyright, MenuMaker.author, MenuMaker.email))
		sys.stdout.write("**** THIS IS OF AN ALPHA CODE QUALITY, NOT FOR PRODUCTION USE ****\n\n")
		_OP.print_help(self)
		sys.stdout.write("\nfrontends (case insensitive):\n")
		for v in fronts.values() :
			vars = list(v)
			s = str(vars[0])
			for x in vars[1:] :
				s += " | " + str(x)
			sys.stdout.write("  %s\n" % s)
		sys.stdout.write("\nskip categories for -s (case insensitive):\n")
		for k in sKwS :
			sys.stdout.write("  %s\n" % k)




op = OP(usage = "%prog [options] frontend", version = "%s %s" % (MenuMaker.pkgName, MenuMaker.pkgVer))
op.add_option("-v", "--verbose", action = "callback", callback = vOpt, help = vStr)
op.add_option("-f", "--force", action = "callback", callback = fOpt, help = fStr)
op.add_option("-c", "--stdout", action = "callback", callback = cOpt, help = cStr)
op.add_option("-i", action = "callback", callback = iOpt, help = iStr)
op.add_option("-r", "--retain-path", action = "callback", callback = rOpt, help = rStr)
op.add_option("--no-desktop", action = "callback", callback = xdOpt, help = xdStr)
op.add_option("--no-legacy", action = "callback", callback = xlOpt, help = xlStr)
op.add_option("-s", "--skip", metavar = "list", nargs = 1, type = "string", action = "callback", callback = sOpt, help = sStr)




(opts, args) = op.parse_args()




if not len(args) :
	fatal("no frontend specified")




found = False
for k, v in fronts.items() :
	if args[0] in v :
		found = True
		name = "MenuMaker.%s" % k
		frontName = k
		__import__(name)
		front = sys.modules[name]
		MenuMaker.setFrontend(front)
		break
if not found :
	fatal("wrong frontend: %s" % args[0])




if len(args) > 1 :
	warn("multiple frontends specified; the first one (%s) will be used" % frontName)




try :
	menuFile = front.menuFile
except AttributeError :
	if not writeToStdout :
		fatal("%s doesn't support writing to a file; use -c option instead" % frontName)




if not MenuMaker.writeFullMenu and not writeToStdout :
	fatal("-i MUST be used in conjunction with -c; this is done to prevent accidental overwriting of the custom menu")




import Prophet.Desktop
import Prophet.Legacy # Postpone the setup to the time when it's actually needed




msg("* scanning")




if noDesktop :
	msg("  skipping desktop")
	desktop = []
else :
	desktop = Prophet.Desktop.scan()




if noLegacy :
	msg("  skipping legacy")
	legacy = []
else :
	Prophet.Legacy.setup()
	legacy  = Prophet.Legacy.scan()




merged = Prophet.merge(legacy + desktop)




msg("* generating")




menu = MenuMaker.Root()
menu.distribute(merged)
menu.arrange()




if writeToStdout :
	msg("* writing")
	file = sys.stdout
else :
	msg("* writing to %s" % menuFile)
	file = os.path.expanduser(menuFile)
	dir = os.path.dirname(file)
	if not os.path.isdir(dir) :
		msg("  creating missing directory %s" % dir)
		os.makedirs(dir)
	if not forceWrite and os.path.isfile(file) :
		fatal("refuse to overwrite existing file %s; either delete it or use -f option" % file)
	file = open(file, "wt")




for x in menu.emit(0) :
	file.write(x)
	file.write("\n")




msg("* done")
